import json
from os.path import join, dirname
from os import getcwd

from web3 import Web3
import web3
import requests

ABI_DIR = join(dirname(__file__), "abi")


def get_web3_provider(network_rpc_url: str) -> Web3:
    return Web3(Web3.HTTPProvider(network_rpc_url))


def get_contract_instance(abi_filename: str, contract_address: str, w3_provider: Web3 = None):
    if w3_provider is None:
        w3_provider = get_web3_provider("https://bsc-dataseed.binance.org/")
        
    with open(join(ABI_DIR, abi_filename), 'r') as abi_file:
        return w3_provider.eth.contract(abi=json.load(abi_file),
                                        address=Web3.toChecksumAddress(contract_address))
    
    
def store_abi(abi_url: str, abi_filename: str) -> None:
    """
    Format and store a smart contract ABI in JSON locally

    :param abi_url: NETWORK SCAN -> Export ABI -> RAW/Text Format -> Get the URL
                    (ex. "http://api.etherscan.io/api?module=contract&action=getabi&address=0xc4a59cfed3fe06bdb5c21de75a70b20db280d8fe&format=raw")
    :param abi_filename: The desired filename for local storage
    """
    contract_abi = requests.get(abi_url).json()
    path = join(join(dirname(__file__)), "abi", abi_filename)
    with open(path, "w") as json_file:
        json_file.write(json.dumps(contract_abi, indent=2))


def checksum(address: str):
    return Web3.toChecksumAddress(address)



from .util import get_contract_instance, checksum

from web3 import Web3
import web3.contract


class BEP20Token:
    """https://github.com/bnb-chain/BEPs/blob/master/BEP20.md"""
    def __init__(self, token_address: str, w3_provider: Web3 = None):
        self.contract = get_contract_instance(abi_filename="BEP20.json",
                                              contract_address=token_address,
                                              w3_provider=w3_provider)
        self.address = token_address

    def name(self) -> str:
        """Returns the name of the token - e.g. 'MyToken'."""
        return self.contract.functions.name().call()

    def symbol(self) -> str:
        """Returns the symbol of the token. E.g. “HIX”."""
        return self.contract.functions.symbol().call()

    def decimals(self) -> int:
        """
        Returns the number of decimals the token uses.
        E.g. 8, means to divide the token amount by 100000000 to get its user representation.
        """
        return self.contract.functions.decimals().call()

    def totalSupply(self) -> int:
        """
        Returns the total token supply. If the token will flow across the Binance Chain and Binance Smart Chain, 
        the number should be the total of circulation across 2 blockchains.
        """
        return self.contract.functions.totalSupply().call()

    def balanceOf(self, _owner: str) -> int:
        """Returns the account balance of an account with address _owner."""
        return self.contract.functions.balanceOf(checksum(_owner)).call()

    def getOwner(self):
        raise NotImplementedError

    def transfer(self, _to: str, _value: int):
        """Transfers _value of the token to _to."""
        raise NotImplementedError

    def transferFrom(self, _from: str, _to: str, _value: int):
        """Transfers _value amount of tokens from address _from to address _to"""
        raise NotImplementedError

    def allowance(self, _owner: str, _spender: str) -> int:
        """Returns the amount which _spender is still allowed to withdraw from _owner"""
        return self.contract.functions.allowance(_owner, _spender).call()
    
    def prepare_approve(self, _spender: str, _value: int = None) -> web3.contract.ContractFunction:
        """
        UNCALLED - Prepares an approve transaction. Should be signed by the owner using web3.
        https://web3py.readthedocs.io/en/stable/web3.eth.html#web3.eth.Eth.sign_transaction
        
        Allows _spender to withdraw from your account multiple times, up to the _value amount.
        If this function is called again it overwrites the current allowance with _value.
        """
        if _value is None:  # if _value is not specified, approve the max amount
            _value = 2 ** 256 - 1
        return self.contract.functions.approve(checksum(_spender), _value)



from setuptools import setup, find_packages

import bep20


def readme():
    with open('README.md') as infile:
        return infile.read().strip()


setup(
    name='bep20',
    version=bep20.__version__,
    author='Harrison Schick',
    author_email='hschickdevs@gmail.com',
    description='Python package that models the methods for tokens following the BEP-20 token standard on the Binance Smart chain.',
    long_description=readme(),
    long_description_content_type="text/markdown",
    url='https://github.com/hschickdevs/Python-BEP20-Token',
    # license='MIT',
    packages=['bep20'],
    include_package_data=True,
    install_requires=[line.strip() for line in open('requirements.txt').readlines()],
)
